"""
    Serializer for Blackboard configuration.
"""
from rest_framework import serializers

from integrated_channels.blackboard.models import BlackboardEnterpriseCustomerConfiguration


class BlackboardConfigSerializer(serializers.ModelSerializer):
    class Meta:
        model = BlackboardEnterpriseCustomerConfiguration
        fields = '__all__'



# Generated by Django 2.2.16 on 2020-10-13 19:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blackboard', '0003_blackboardlearnerdatatransmissionaudit'),
    ]

    operations = [
        migrations.AlterField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
        migrations.AlterField(
            model_name='historicalblackboardenterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Admin utilities.
"""

import unicodecsv

from django.contrib import auth
from django.core.exceptions import ValidationError
from django.core.paginator import EmptyPage, PageNotAnInteger

from enterprise.admin.paginator import CustomPaginator
from enterprise.utils import ValidationMessages

DOT = '.'
PAGES_ON_EACH_SIDE = 3
PAGES_ON_ENDS = 2
User = auth.get_user_model()


class UrlNames:
    """
    Collection on URL names used in admin
    """
    URL_PREFIX = "enterprise_"
    MANAGE_LEARNERS = URL_PREFIX + "manage_learners"
    MANAGE_LEARNERS_DSC = URL_PREFIX + "manage_learners_data_sharing_consent"
    TRANSMIT_COURSES_METADATA = URL_PREFIX + "transmit_courses_metadata"
    PREVIEW_EMAIL_TEMPLATE = URL_PREFIX + "preview_email_template"


def validate_csv(file_stream, expected_columns=None):
    """
    Validate csv file for encoding and expected header.

    Args:
        file_stream: input file
        expected_columns: list of column names that are expected to be present in csv

    Returns:
       reader: an iterable for csv datat if csv passes the validation

    Raises:
        ValidationError
    """
    try:
        reader = unicodecsv.DictReader(file_stream, encoding="utf-8-sig")
        reader_fieldnames = reader.fieldnames
    except (unicodecsv.Error, UnicodeDecodeError) as error:
        raise ValidationError(ValidationMessages.INVALID_ENCODING) from error

    if expected_columns and set(expected_columns) - set(reader_fieldnames):
        raise ValidationError(ValidationMessages.MISSING_EXPECTED_COLUMNS.format(
            expected_columns=", ".join(expected_columns), actual_columns=", ".join(reader.fieldnames)
        ))

    return reader


def parse_csv(file_stream, expected_columns=None):
    """
    Parse csv file and return a stream of dictionaries representing each row.

    First line of CSV file must contain column headers.

    Arguments:
         file_stream: input file
         expected_columns (set[unicode]): columns that are expected to be present

    Yields:
        dict: CSV line parsed into a dictionary.
    """
    reader = validate_csv(file_stream, expected_columns)
    yield from reader


def email_or_username__to__email(email_or_username):
    """
    Convert email_or_username to email.

    Returns:
        str: If `email_or_username` was a username returns user's email, otherwise assumes it was an email and returns
             as is.
    """
    try:
        user = User.objects.get(username=email_or_username)
        return user.email
    except User.DoesNotExist:
        return email_or_username


def split_usernames_and_emails(email_field):
    """
    Split the contents of the email field into a list.

    In some cases, a user could enter a comma-separated value inline in the
    Manage Learners form. We should check to see if that's the case, and
    provide a list of email addresses or usernames if it is.
    """
    return [name.strip() for name in email_field.split(',')]


# pylint: disable=range-builtin-not-iterating
def paginated_list(object_list, page, page_size=25):
    """
    Returns paginated list.

    Arguments:
        object_list (QuerySet): A list of records to be paginated.
        page (int): Current page number.
        page_size (int): Number of records displayed in each paginated set.
        show_all (bool): Whether to show all records.

    Adopted from django/contrib/admin/templatetags/admin_list.py
    https://github.com/django/django/blob/1.11.1/django/contrib/admin/templatetags/admin_list.py#L50
    """
    paginator = CustomPaginator(object_list, page_size)
    try:
        object_list = paginator.page(page)
    except PageNotAnInteger:
        object_list = paginator.page(1)
    except EmptyPage:
        object_list = paginator.page(paginator.num_pages)

    page_range = []
    page_num = object_list.number

    # If there are 10 or fewer pages, display links to every page.
    # Otherwise, do some fancy
    if paginator.num_pages <= 10:
        page_range = range(paginator.num_pages)
    else:
        # Insert "smart" pagination links, so that there are always ON_ENDS
        # links at either end of the list of pages, and there are always
        # ON_EACH_SIDE links at either end of the "current page" link.
        if page_num > (PAGES_ON_EACH_SIDE + PAGES_ON_ENDS + 1):
            page_range.extend(range(1, PAGES_ON_ENDS + 1))
            page_range.append(DOT)
            page_range.extend(range(page_num - PAGES_ON_EACH_SIDE, page_num + 1))
        else:
            page_range.extend(range(1, page_num + 1))
        if page_num < (paginator.num_pages - PAGES_ON_EACH_SIDE - PAGES_ON_ENDS):
            page_range.extend(range(page_num + 1, page_num + PAGES_ON_EACH_SIDE + 1))
            page_range.append(DOT)
            page_range.extend(range(paginator.num_pages + 1 - PAGES_ON_ENDS, paginator.num_pages + 1))
        else:
            page_range.extend(range(page_num + 1, paginator.num_pages + 1))

        # Override page range to implement custom smart links.
        object_list.paginator.page_range = page_range

    return object_list



# -*- coding: utf-8 -*-
"""
Enterprise Integrated Channel Django application initialization.
"""

from django.apps import AppConfig


class IntegratedChannelConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel Django application.
    """
    name = 'integrated_channels.integrated_channel'
    verbose_name = "Enterprise Integrated Channels"



"""
    url mappings for integrated_channels/v1/blackboard/
"""

from rest_framework import routers

from .views import BlackboardConfigurationViewSet

app_name = 'blackboard'
router = routers.DefaultRouter()  # pylint: disable=invalid-name
router.register(r'configuration', BlackboardConfigurationViewSet, basename="configuration")
urlpatterns = router.urls



# -*- coding: utf-8 -*-
"""
Django management command for sending an email to learners with missing DataSharingConsent records.
"""
import datetime
import logging
from datetime import date, timedelta
from urllib.parse import urljoin

from django.conf import settings
from django.core.management import BaseCommand
from django.urls import reverse

from consent.models import DataSharingConsent, ProxyDataSharingConsent
from enterprise import utils
from enterprise.api_client.discovery import CourseCatalogApiClient
from enterprise.api_client.lms import parse_lms_api_datetime
from enterprise.models import EnterpriseCourseEnrollment
from enterprise.utils import get_configuration_value

try:
    from openedx.features.enterprise_support.utils import is_course_accessed
except ImportError:
    is_course_accessed = None

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Django management command for sending an email to learners with missing DataSharingConsent records
    """

    def _get_course_properties(self, user, course_id, enterprise_customer):
        """
        Provide the data sharing consent page next url and course title

        Arguments
            user (Object): The User object
            course_id (String): The course identifier
            enterprise_customer(Object): EnterpriseCustomer object

        Returns:
            next_url (String): The DSC url 'next' param
            course_title (String): Title of the course
        """
        learner_portal_base_url = get_configuration_value(
            'ENTERPRISE_LEARNER_PORTAL_BASE_URL',
            settings.ENTERPRISE_LEARNER_PORTAL_BASE_URL
        )
        next_url = urljoin(learner_portal_base_url, str(enterprise_customer.slug))
        course_start = None
        course_title = ''
        course_details = CourseCatalogApiClient(
            user,
            enterprise_customer.site
        ).get_course_run(course_id)
        if course_details:
            try:
                course_start = parse_lms_api_datetime(course_details.get('start'))
                course_title = course_details.get('title')
            except (TypeError, ValueError):
                pass
        else:
            LOGGER.info(
                '[Absent DSC Course Details] Could not get course details from course catalog API. '
                'User: [%s], Course: [%s], Enterprise: [%s]',
                user.username,
                course_id,
                enterprise_customer.uuid,
            )

        if course_start and course_start < datetime.datetime.now(course_start.tzinfo):
            lms_course_url = urljoin(settings.LMS_ROOT_URL, '/courses/{course_id}/course')
            next_url = lms_course_url.format(course_id=course_id)

        return next_url, course_title

    def get_enterprise_course_enrollments(self, options):
        """
        Get EnterpriseCourseEnrollment records according to the options and with dsc enabled
        """
        enrollment_before = options['enrollment_before']
        past_num_days = options['past_num_days']
        if enrollment_before:
            enrollments_with_dsc_enabled = EnterpriseCourseEnrollment.objects.select_related(
                'enterprise_customer_user').filter(
                created__date__lt=enrollment_before,
                enterprise_customer_user__enterprise_customer__enable_data_sharing_consent=True
            )
        else:
            past_date = date.today() - timedelta(days=past_num_days)
            enrollments_with_dsc_enabled = EnterpriseCourseEnrollment.objects.select_related(
                'enterprise_customer_user').filter(
                created__date=past_date,
                enterprise_customer_user__enterprise_customer__enable_data_sharing_consent=True
            )

        return enrollments_with_dsc_enabled

    def emit_event(self, ec_user, course_id, enterprise_customer, greeting_name):
        """
         Emit the Segment event which will be used by Braze to send the email
        """
        next_url, course_title = self._get_course_properties(ec_user.user, course_id, enterprise_customer)
        failure_url = urljoin(settings.LMS_ROOT_URL, '/dashboard')
        grant_data_sharing_url = reverse('grant_data_sharing_permissions')
        utils.track_event(ec_user.user_id, 'edx.bi.user.consent.absent', {
            'course_id': course_id,
            'username': ec_user.username,
            'enterprise_name': enterprise_customer.name,
            'enterprise_uuid': str(enterprise_customer.uuid),
            'next_url': next_url,
            'course_title': course_title,
            'user_email': ec_user.user_email,
            'greeting_name': greeting_name,
            'failure_url': failure_url,
            'lms_root_url': settings.LMS_ROOT_URL,
            'grant_data_sharing_url': grant_data_sharing_url
        })
        LOGGER.info(
            '[Absent DSC Email] Segment event fired for missing data sharing consent. '
            'User: [%s], Course: [%s], Enterprise: [%s], DSC Next Url: [%s]',
            ec_user.username,
            course_id,
            str(enterprise_customer.uuid),
            next_url
        )

    def add_arguments(self, parser):
        parser.add_argument(
            '--no-commit',
            action='store_true',
            dest='no_commit',
            default=False,
            help='Dry Run, print log messages without committing anything.',
        )
        parser.add_argument(
            '--enrollment-before',
            action='store',
            dest='enrollment_before',
            default=None,
            type=datetime.date.fromisoformat,
            help='Specifies the date (format YYYY-MM-DD). Enrollments created before this date will receive DSC emails.'
        )
        parser.add_argument(
            '--past_num_days',
            action='store',
            dest='past_num_days',
            default=1,
            type=int,
            help='Days past the current day. Enrollments created on that day with a missing DSC will be processed.'
        )

    def handle(self, *args, **options):
        """
        Management command for sending an email to learners with a missing DSC. Designed to run daily.

        Example usage:
           $ ./manage.py email_drip_for_missing_dsc_records
           $ ./manage.py email_drip_for_missing_dsc_records  --no-commit
           $ ./manage.py email_drip_for_missing_dsc_records  --enrollment-before 2021-05-06 --no-commit
           $ ./manage.py email_drip_for_missing_dsc_records  --enrollment-before 2021-05-06
        """
        should_commit = not options['no_commit']
        email_sent_records = []
        enterprise_course_enrollments = self.get_enterprise_course_enrollments(options)
        for enterprise_enrollment in enterprise_course_enrollments:
            ec_user = enterprise_enrollment.enterprise_customer_user
            username = ec_user.username
            user_email = ec_user.user_email
            greeting_name = user_email
            if hasattr(ec_user, 'first_name') and ec_user.first_name:
                greeting_name = ec_user.first_name
            course_id = enterprise_enrollment.course_id
            enterprise_customer = ec_user.enterprise_customer
            consent = DataSharingConsent.objects.proxied_get(
                username=username,
                course_id=course_id,
                enterprise_customer=enterprise_customer
            )
            if isinstance(consent, ProxyDataSharingConsent):
                course_accessed = False
                try:
                    if is_course_accessed and is_course_accessed(ec_user.user, course_id):
                        course_accessed = True
                except Exception as exc:  # pylint: disable=broad-except
                    LOGGER.exception('[Absent DSC Email] Error in {course} for user {user}. Error detail: {exc}'.format(
                        course=course_id,
                        user=username,
                        exc=str(exc)
                    ))
                # Emit the Segment event which will be used by Braze to send the email
                if course_accessed:
                    if should_commit:
                        self.emit_event(ec_user, course_id, enterprise_customer, greeting_name)
                    email_sent_records.append(
                        f'User: {username}, Course: {course_id}, Enterprise: {enterprise_customer.uuid}'
                    )
        LOGGER.info(
            '[Absent DSC Email] Emails sent for [%s] enrollments out of [%s] enrollments. DSC records sent to: [%s]',
            len(email_sent_records),
            enterprise_course_enrollments.count(),
            email_sent_records
        )



# -*- coding: utf-8 -*-
"""
The Canvas Integrated Channel package.
"""

__version__ = "0.0.1"

default_app_config = ("integrated_channels.canvas.apps."  # pylint: disable=invalid-name
                      "CanvasConfig")



# -*- coding: utf-8 -*-
"""
Enterprise Integrated Channel Moodle Django application initialization.
"""

from django.apps import AppConfig


class MoodleConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel Moodle Django application.
    """
    name = 'integrated_channels.moodle'
    verbose_name = 'Enterprise Moodle Integration'



# -*- coding: utf-8 -*-
"""
Errors thrown by the APIs in the Enterprise application.
"""


class CodesAPIRequestError(Exception):
    """There was a problem with a request to the Codes application's APIs."""


class EnrollmentModificationException(Exception):
    """
    An exception that represents an error when modifying the state
    of an enrollment via the EnrollmentApiClient.
    """


class AdminNotificationAPIRequestError(Exception):
    """
    An exception that represents an error when creating admin notification
    read status via the NotificationReadApiClient.
    """
